//
//  MainRoutine.m
//  MINtS analyzer
//
//  Created by 萩原 弘一 on 2013/09/06.
//  Copyright (c) 2013年 萩原 弘一. All rights reserved.
//
/* Copyright 2016 Koichi Hagiwara
 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import "MainRoutine.h"

@implementation MainRoutine
@synthesize mainMutDictionary;
@synthesize summaryForEachPatientMutAr;
@synthesize selectedPatientRowNo;
@synthesize summaryForEachPatient;
@synthesize expandedTemporaryFilesFullPathAr;
@synthesize mainDocument;
@synthesize mutDomains;

//total reads -> informative reads + unoverlapped
//informative reads -> mapped and unmapped
//unoverlapped -> mapped and unmapped

//fileAr contains filepath
-( id )initWithDocument:( MintsDocument * )document {
    self = [ super init ];
    if( self ) {
        // summaryForEachPatientMutAr has a single object
        mainMutDictionary = [ NSMutableDictionary dictionaryWithCapacity:NUMBER_OF_PATIENTS_PER_KIT ];
        summaryForEachPatientMutAr = [ [ NSMutableArray alloc ] initWithCapacity:1 ];
        resultCalc = [ [ ResultCalc alloc ] init ];
        resultCalc.mainRoutine = self;
        mainDocument = document;
        mutDomains = [ self setUpMutDomainsMutAr ];
        summaryForEachPatient = [ [ SummaryForEachPatient alloc ] initWithMutDomains:mutDomains ];
        selectedPatientRowNo = -1;  // Unselect patientRowNoTextView
    }
    return self;
}

-( BOOL )execute {
    [ mainDocument.windowController.file1TextField setStringValue:summaryForEachPatient.file1 ];
    [ mainDocument.windowController.file2TextField setStringValue:summaryForEachPatient.file2 ];

//    NSString *tFastqR1R2File;
    // mergeR1R2...@"NO" or fastqR1R2file
    if( [ self prepareSeqR1andR2andCallPickupMatches ] == NO ) {
        return NO;
    }
    [ mainDocument.windowController.totalNumberOfReadsTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.totalNumberOfReads ] ];
    [ mainDocument.windowController.readsMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.readsMapped ] ];
    [ mainDocument.windowController.readsFailedToBeMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.readsFailedToBeMapped ] ];
    [ self summarizeResult ];
    // at this point all data for each patient is ready for storing
    [ self cleanupFiles ];
    return YES;
}

-( void )cleanupFiles {
    // Delete temporary files
    NSUserDefaults *defaults = [ NSUserDefaults standardUserDefaults ];
    NSFileManager *tFileManager = [ NSFileManager defaultManager ];
    NSArray *tAr = [ expandedTemporaryFilesFullPathAr[ 0 ] componentsSeparatedByString:@"_R1" ];
    [ tFileManager removeItemAtPath:[ NSString stringWithFormat:@"%@_R1_001.fastq", tAr[ 0 ] ] error:nil ];
    [ tFileManager removeItemAtPath:[ NSString stringWithFormat:@"%@_R2_001.fastq", tAr[ 0 ] ] error:nil ];
    if( [ defaults valueForKey:@"leaveFileForFailedToBeAssignedReads" ] != [ NSNumber numberWithBool:YES ] ){
        [ tFileManager removeItemAtPath:[ NSString stringWithFormat:@"%@.failedToBeAssignedReads", tAr[ 0 ] ] error:nil ];
//        NSLog(@"%@.failedToBeAssignedReads", tAr[ 0 ]);
    }
}

-( BOOL )prepareSeqR1andR2andCallPickupMatches {
    MutDomain *tMutDomain;
    Mutation *tMutation;
    FILE *outFailedToBeAssignedReads;
    unsigned long i, j;
    
    // initialization
    summaryForEachPatient.readsFailedToBeMapped = 0;
    summaryForEachPatient.readsMapped = 0;
    summaryForEachPatient.totalNumberOfReads = 0;
    [ mainDocument.windowController.readsFailedToBeMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.readsFailedToBeMapped ] ];
    [ mainDocument.windowController.readsMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.readsMapped ] ];
    [ mainDocument.windowController.totalNumberOfReadsTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.totalNumberOfReads ] ];

    
    for( i = 0; i < [ summaryForEachPatient.mutDomains count ]; ++i ){
        tMutDomain = summaryForEachPatient.mutDomains[ i ];
        for( j = 0; j < [ tMutDomain.mutationMutAr count ]; ++j ) {
            tMutation = tMutDomain.mutationMutAr[ j ];
            tMutation.match = 0;
        }
    }

    //--------------------
    // for returning
    NSArray *tAr = [ expandedTemporaryFilesFullPathAr[ 0 ] componentsSeparatedByString:@"_R1_" ];

    char *buf2, *header1, *header2, *buf11, *buf21, *buf12, *buf22, *buf13, *buf23;
    buf2 = ( char * )malloc( R1SEQLENGTH + 1 );
    buf11 = ( char * )malloc( R1SEQLENGTH + 1 );
    buf21 = ( char * )malloc( R1SEQLENGTH + 1 );
    buf12 = ( char * )malloc( R1SEQLENGTH + 1 );
    buf22 = ( char * )malloc( R1SEQLENGTH + 1 );
    buf13 = ( char * )malloc( R1SEQLENGTH + 1 );
    buf23 = ( char * )malloc( R1SEQLENGTH + 1 );
    header1 = ( char * )malloc( R1SEQLENGTH + 1 );
    header2 =  ( char * )malloc( R1SEQLENGTH + 1 );
    FILE *in1, *in2;

    const char *infile1 = [ expandedTemporaryFilesFullPathAr[ 0 ] UTF8String ];
    const char *infile2 = [ expandedTemporaryFilesFullPathAr[ 1 ] UTF8String ];
    in1 = fopen( infile1, "r" );
    in2 = fopen( infile2, "r" );
    const char *unassignedReadFilePath = [ [ NSString stringWithFormat:@"%@.failedToBeAssignedReads", tAr[ 0 ] ] UTF8String ];
    outFailedToBeAssignedReads = fopen( unassignedReadFilePath, "w" );
    if( outFailedToBeAssignedReads == NULL ) {
        [ mainDocument displayError:[ NSString stringWithFormat:@"Cannot open1 %@.\n", [ NSString stringWithUTF8String:unassignedReadFilePath ] ] ];
        return NO;
    }

    if( in1 == NULL || in2 == NULL ) {
        if( in1 == NULL ) {
            [ mainDocument displayError:[ NSString stringWithFormat:@"Cannot open3 %@.\n", [ NSString stringWithUTF8String:infile1 ] ] ];
        } else if( in2 == NULL ) {
            [ mainDocument displayError:[ NSString stringWithFormat:@"Cannot open4 %@.\n", [ NSString stringWithUTF8String:infile2 ] ] ];
        }
        return NO;// @"NO";
    }
    
    char *tCPtr1_0, *tCPtr2_0, *tCPtr1_1, *tCPtr2_1, *tCPtr1_2, *tCPtr2_2, *tCPtr1_3, *tCPtr2_3;
    
    int cycle = 0; // for debug
    dispatch_queue_t myQueue = dispatch_queue_create( "com.hagiwark", NULL ); // Serial queue
    dispatch_queue_t globalQueue = dispatch_get_global_queue( DISPATCH_QUEUE_PRIORITY_DEFAULT, 0 );
    dispatch_group_t dispatchGroup = dispatch_group_create();
    while( 1 ) {
        // for freeing in a branched thread
        char **header, **r1Seq, **r2Seq, **r1Qual, **r2Qual;
        myFgets( buf2, R1SEQLENGTH, in1 );  // Read first line.
        if( buf2[ 0 ] == 0 ){ // If all files have been read to the end, goto nextStep.
            goto nextStep;
        }
        header = ( char ** )calloc( NUMBER_OF_READS_PER_CHUNK, sizeof( char* ) );
        r1Seq = ( char ** )calloc( NUMBER_OF_READS_PER_CHUNK, sizeof( char* ) );
        r2Seq = ( char ** )calloc( NUMBER_OF_READS_PER_CHUNK, sizeof( char* ) );
        r1Qual = ( char ** )calloc( NUMBER_OF_READS_PER_CHUNK, sizeof( char* ) );
        r2Qual = ( char ** )calloc( NUMBER_OF_READS_PER_CHUNK, sizeof( char* ) );
        for( i = 0; i < NUMBER_OF_READS_PER_CHUNK; ++i ) {
            *( header + i ) = ( char * )malloc( R1SEQLENGTH + 1 );
            *( r1Seq + i ) = ( char * )malloc( R1SEQLENGTH + 1 );
            *( r2Seq + i ) = ( char * )malloc( R1SEQLENGTH + 1 );
            *( r1Qual + i ) = ( char * )malloc( R1SEQLENGTH + 1 );
            *( r2Qual + i ) = ( char * )malloc( R1SEQLENGTH + 1 );
        }
        for( i = 0; i < NUMBER_OF_READS_PER_CHUNK; ++i ) {
            if( i == 0 ){
                // The first line has already been in buf2.
                strcpy( header1, buf2 ); tCPtr1_0 = buf2;
            } else {
                tCPtr1_0 = myFgets( header1, R1SEQLENGTH, in1 );
            }
            tCPtr2_0 = myFgets( header2, R1SEQLENGTH, in2 );
            // If the end of file is reached, do not read more.
            if( ( tCPtr1_0 == NULL ) || ( tCPtr2_0 == NULL ) ){
                for( j = i; j < NUMBER_OF_READS_PER_CHUNK; ++j ) {
                    *( *( header + j ) ) = 0;
                    *( *( r1Seq + j ) ) = 0;
                    *( *( r2Seq + j ) ) = 0;
                    *( *( r1Qual + j ) ) = 0;
                    *( *( r2Qual + j ) ) = 0;
                }
                break;
            }
            for( j = 0; j < strlen( header1 ); ++j ){
                // Truncate each header at the first space
                if( header1[ j ] == ' ' ){ header1[ j ] = 0; }
                if( header2[ j ] == ' ' ){ header2[ j ] = 0; }
            }
            // If the headers do not match, post a message and exit.
            if( strcmp( header1, header2 ) != 0 ){
                [ mainDocument displayError:[ NSString stringWithFormat:@"Two fastq files do not match at %s.\n", header1 ] ];
                return NO;// @"NO";
            }
            strcpy( *( header + i ), header1 );
            tCPtr1_1 = myFgets( buf11, R1SEQLENGTH, in1 ); strcpy( *( r1Seq + i ), buf11 );
            tCPtr2_1 = myFgets( buf21, R1SEQLENGTH, in2 ); strcpy( *( r2Seq + i ), buf21 );
            tCPtr1_2 = myFgets( buf12, R1SEQLENGTH, in1 );
            tCPtr2_2 = myFgets( buf22, R1SEQLENGTH, in2 );
            tCPtr1_3 = myFgets( buf13, R1SEQLENGTH, in1 ); strcpy( *( r1Qual + i ), buf13 );
            tCPtr2_3 = myFgets( buf23, R1SEQLENGTH, in2 ); strcpy( *( r2Qual + i ), buf23 );
            
            if( tCPtr1_1 == NULL || tCPtr2_1 == NULL || tCPtr1_2 == NULL || tCPtr2_2 == NULL || tCPtr1_3 == NULL || tCPtr2_3 == NULL ) {
                [ mainDocument displayError:[ NSString stringWithFormat:@"%@ or %@ are trimcated/\n", expandedTemporaryFilesFullPathAr[ 0 ], expandedTemporaryFilesFullPathAr[ 0 ] ] ];
                return NO;// @"NO";
            }
        }
        for( i = 0; i < NUMBER_OF_READS_PER_CHUNK; ++i ) {
            rcC( *( r2Seq + i ) );
            reverseC( *( r2Qual + i ) );
        }
        dispatch_group_async( dispatchGroup, globalQueue, ^{
            @autoreleasepool {
                [ self pickupMatchesWithCoreSeq:header R1Seq:r1Seq R2Seq:r2Seq R1Qual:r1Qual R2Qual:r2Qual file:outFailedToBeAssignedReads queue: myQueue ];
            }
        } );
        cycle++;
        if( cycle % 8 == 0 ){
            dispatch_group_wait( dispatchGroup, DISPATCH_TIME_FOREVER );
        }
    }
nextStep:
    dispatch_group_wait( dispatchGroup, DISPATCH_TIME_FOREVER );
    dispatch_sync( myQueue, ^{} );
    fclose( in1 ); fclose( in2 ); fclose( outFailedToBeAssignedReads );
    free( buf2 ); free( header1 ); free( header2 ); free( buf11 ); free( buf21 ); free( buf12 ); free( buf22 ); free( buf13 ); free( buf23 );
    dispatch_release( myQueue );
    dispatch_release( dispatchGroup );

    //----------------

    for( i = 0; i < [ summaryForEachPatient.mutDomains count ]; ++i ) {
        tMutDomain = summaryForEachPatient.mutDomains[ i ];
        tMutDomain.totalNumberOfMutantReads = 0;
        for( j = 0; j < [ tMutDomain.mutationMutAr count ]; ++j ) {
            tMutation = tMutDomain.mutationMutAr[ j ];
            if( j == 0 ) {
                tMutDomain.totalNumberOfNormalReads = tMutation.match;
            } else {
                tMutDomain.totalNumberOfMutantReads += tMutation.match;
            }
        }
    }
    summaryForEachPatient.readsMapped = summaryForEachPatient.totalNumberOfReads - summaryForEachPatient.readsFailedToBeMapped;
    [ mainDocument.windowController.totalNumberOfReadsTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.totalNumberOfReads ] ];
    [ mainDocument.windowController.readsFailedToBeMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.readsFailedToBeMapped ] ];
    [ mainDocument.windowController.readsMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.readsMapped ] ];
    return YES;
}

-( void )pickupMatchesWithCoreSeq:( char ** )header R1Seq:( char ** )r1Seq R2Seq:( char ** )r2Seq R1Qual:( char ** )r1Qual R2Qual:( char ** )r2Qual file:( FILE * )outFailedToBeAssignedReads queue:( dispatch_queue_t ) myQueue {
    // This routine frees r1Seq, r2Seq, r1Qual and r2Qual.
    unsigned long i, j, k;
    int count = 0;
    for( i = 0; i < [ summaryForEachPatient.mutDomains count ]; ++i ) {
        MutDomain *tMutDomain = summaryForEachPatient.mutDomains[ i ];
        count += [ tMutDomain.mutationMutAr count ];
    }
    long *match = ( long * )calloc( count, sizeof( long ) );
    NSInteger tReadsFailedToBeMapped = 0;
    NSInteger tTotalNumberOfReads = 0;
    NSMutableString *failedToBeAssignedReads = [ [ NSMutableString alloc ] initWithString:@"" ];
    for( k = 0; k < NUMBER_OF_READS_PER_CHUNK; ++k ) {
        if( *( *( r1Seq + k ) ) != 0 && *( *( r2Seq + k ) ) != 0 ) {    // If r1Seq and r2Seq are not empty
            tTotalNumberOfReads++;
            BOOL flag = NO;
            char *buf1 = *( r1Seq + k );
            char *buf2 = *( r2Seq + k );
            char *qbuf1 = *( r1Qual + k );
            char *qbuf2 = *( r2Qual + k );
            MutDomain *tMutDomain;
            Mutation *tMutation;
            for( count = i = 0; i < [ summaryForEachPatient.mutDomains count ]; ++i ) {
                tMutDomain = summaryForEachPatient.mutDomains[ i ];
                for( j = 0; j < [ tMutDomain.mutationMutAr count ]; ++j ) {
                    tMutation = tMutDomain.mutationMutAr[ j ];
                    if( [ tMutation.strandRequiredToRead isEqualToString:@"R1R2" ] ){
                        if( tMutation.coreSeqContainsWildCard == NO ) {
#ifdef CONSIDER_PHREDSCORE
                            if( nucStrstrT( buf1, tMutation.coreSeqC, qbuf1, '@' ) && nucStrstrT( buf2, tMutation.coreSeqC, qbuf2, '@' ) ) {
#else
                            if( nucStrstr( buf1, tMutation.coreSeqC ) && nucStrstr( buf2, tMutation.coreSeqC ) ) {
#endif
                                match[ count + j ]++;
                                flag = YES; break;
                            }
                        } else if( tMutation.coreSeqContainsWildCard == YES ) {
#ifdef CONSIDER_PHREDSCORE
                            if( nucStrstrWildCardT( buf1, tMutation.coreSeqC, qbuf1, '@' ) && nucStrstrWildCardT( buf2, tMutation.coreSeqC, qbuf1, '@' ) ) {
#else
                            if( nucStrstrWildCard( buf1, tMutation.coreSeqC ) && nucStrstrWildCard( buf2, tMutation.coreSeqC ) ){
#endif
                                match[ count + j ]++;
                                flag = YES; break;
                            }
                        }
                    } else if( [ tMutation.strandRequiredToRead isEqualToString:@"R1" ] ){
                        if( tMutation.coreSeqContainsWildCard == NO ) {
#ifdef CONSIDER_PHREDSCORE
                            if( nucStrstrT( buf1, tMutation.coreSeqC, qbuf1, '@' ) ) {
#else
                            if( nucStrstr( buf1, tMutation.coreSeqC ) ) {
#endif
                                match[ count + j ]++;
                                flag = YES; break;
                            }
                        } else if( tMutation.coreSeqContainsWildCard == YES ) {
#ifdef CONSIDER_PHREDSCORE
                            if( nucStrstrWildCardT( buf1, tMutation.coreSeqC, qbuf1, '@' ) ) {
#else
                            if( nucStrstrWildCard( buf1, tMutation.coreSeqC ) ){
#endif
                                match[ count + j ]++;
                                flag = YES; break;
                            }
                        }
                    } else if( [ tMutation.strandRequiredToRead isEqualToString:@"R2" ] ){
                        if( tMutation.coreSeqContainsWildCard == NO ) {
#ifdef CONSIDER_PHREDSCORE
                            if( nucStrstrT( buf2, tMutation.coreSeqC, qbuf2, '@' ) ) {
#else
                            if( nucStrstr( buf2, tMutation.coreSeqC ) ) {
#endif
                                match[ count + j ]++;
                                flag = YES; break;
                            }
                        } else if( tMutation.coreSeqContainsWildCard == YES ) {
#ifdef CONSIDER_PHREDSCORE
                            if( nucStrstrWildCardT( buf2, tMutation.coreSeqC, qbuf2, '@' ) ) {
#else
                            if( nucStrstrWildCard( buf2, tMutation.coreSeqC ) ) {
#endif
                                match[ count + j ]++;
                                flag = YES; break;
                            }
                        }
                    } else {
//                        NSLog( @"%@\t%@\t%@\t%@\n", tMutation.name, tMutation.before10bp, tMutation.after10bp, tMutation.strandRequiredToRead );
                    }
                }
                
                if( ( j != [ tMutDomain.mutationMutAr count ] ) && ( tMutDomain.canStopSearchIfMatched ) ) {
                    break;
                }
                count += [ tMutDomain.mutationMutAr count ];
            }
            if( flag == NO ){
                [ failedToBeAssignedReads appendString:[ NSString stringWithFormat:@"%s\n", buf1 ] ];
                [ failedToBeAssignedReads appendString:[ NSString stringWithFormat:@"%s\n", buf2 ] ];
                tReadsFailedToBeMapped++;
            }
        }
    }
    //for(i = 0 ; i < count; ++i )printf( "%ld ", match[ i ] );
    dispatch_async( myQueue,^{
        summaryForEachPatient.readsFailedToBeMapped += tReadsFailedToBeMapped;
        summaryForEachPatient.totalNumberOfReads += tTotalNumberOfReads;
        int count = 0; int m, n;
        for( m = 0; m < [ summaryForEachPatient.mutDomains count ]; ++ m ){
            MutDomain *tMutDomain = summaryForEachPatient.mutDomains[ m ];
            for( n = 0; n < [ tMutDomain.mutationMutAr count ]; ++ n ){
                Mutation *tMutation = tMutDomain.mutationMutAr[ n ];
                tMutation.match += match[ count + n ];
            }
            count += [ tMutDomain.mutationMutAr count ];
        }
        if( ![ failedToBeAssignedReads isEqualToString:@"" ] ) {
            fprintf( outFailedToBeAssignedReads, "%s", [ failedToBeAssignedReads UTF8String ] );
        }
        free( match );
    });
    for( i = 0; i < NUMBER_OF_READS_PER_CHUNK; ++i ) {
        if( *( header + i ) != NULL ) { free( *( header + i ) ); };
        if( *( r1Seq + i ) != NULL ) { free( *( r1Seq + i ) ); };
        if( *( r2Seq + i ) != NULL ) { free( *( r2Seq + i ) ); };
        if( *( r1Qual + i ) != NULL ) { free( *( r1Qual + i ) ); };
        if( *( r2Qual + i ) != NULL ) { free( *( r2Qual + i ) ); };
    }
    if( summaryForEachPatient.totalNumberOfReads % 1000 == 0 ){
        [ mainDocument.windowController.totalNumberOfReadsTextField setStringValue:[ NSString stringWithFormat:@"%ld", summaryForEachPatient.totalNumberOfReads ] ];
        [ mainDocument.windowController.readsMappedTextField setStringValue:@"0" ];
        [ mainDocument.windowController.readsFailedToBeMappedTextField setStringValue:@"0" ];
    }
}

-( void )summarizeResult {
    NSInteger i;
    for( i = 0; i < [ summaryForEachPatient.mutDomains count ]; ++i ) {
        MutDomain *tMutdomain = summaryForEachPatient.mutDomains[ i ];
        [ resultCalc calculateResult:tMutdomain ];
    }
    [ resultCalc summarizeResultForPatient:summaryForEachPatient.patientNo mutDomains:summaryForEachPatient.mutDomains summaryForEachPatient:summaryForEachPatient ]; //Summarize reads info stored into mutdomain and define wt, mut, positive or negative
    //Information on reads is summarized in mutDomains.
    //Summarize result into gemlineDNAResultMutAr, somaticDNAResultMutAr, germlineRNAResultMutAr and somaticRNAResultMutAr and attach to summaryForEachPatient.
}

-( NSMutableArray *)setUpMutDomainsMutAr {
    NSMutableArray *tMutDomainMutAr = [ NSMutableArray arrayWithCapacity:50 ];
    FILE *inf;
    char entry[ 100000 ];
    char item0[ 1000 ], item1[ 1000 ], item2[ 1000 ], item3[ 1000 ];
    double dItem0, dItem1, dItem2;
    long lItem0;
    MutDomain *tMutDomain = nil;
    Mutation *tMutation = nil;
    inf = fopen( [ [ [ NSBundle mainBundle ] pathForResource:@"ReferenceNucleotideSequence" ofType:@"txt" ] UTF8String ], "r" );
    int endFlag = 0;
    while( myFgets( entry, 999, inf ) ) {
        switch ( entry[ 0 ] ) {
		case '*':
            item0[ 0 ] = item1[ 0 ] = item2[ 0 ] = 0;
            tMutDomain = [ [ MutDomain alloc ] init ];
            sscanf( entry + 1, "%s %s %s", item0, item1, item2 );
            tMutDomain.name = [ NSString stringWithUTF8String:item0 ];
            tMutDomain.type = [ NSString stringWithUTF8String:item1 ];
             //   NSLog( @"%@\n",tMutDomain.type);
            if( strcasecmp( item2, "StopSearchIfMatched" ) == 0 ) {
                tMutDomain.canStopSearchIfMatched = YES;
            } else {
                tMutDomain.canStopSearchIfMatched = NO;
            }
            tMutDomain.mutationMutAr = [ NSMutableArray arrayWithCapacity:3 ];
            [ tMutDomainMutAr addObject:tMutDomain ];
            break;
			
		case '>':
            dItem0 = dItem1 = dItem2 = 0.0; lItem0 = 0L;
            tMutation = [ [ Mutation alloc ] init ];
            sscanf( entry + 1, "%s %s %s %s %lf %lf %lf %ld", item0, item1, item2, item3, &dItem0, &dItem1, &dItem2, &lItem0 );
            tMutation.name = [ NSString stringWithUTF8String:item0 ];
            tMutation.before10bp = [ NSString stringWithUTF8String:item1 ];
            tMutation.after10bp = [ NSString stringWithUTF8String:item2 ];
            tMutation.strandRequiredToRead = [ NSString stringWithUTF8String:item3 ];
            tMutation.referenceFrequencyOfTheMutantReads = dItem0;
            tMutation.allowableFalsePositiveRate = dItem1;
            tMutation.allowableFalseNegativeRate = dItem2;
            tMutation.minimumRequiredNumberOfInformativeReads = lItem0;
            myFgets( entry, 999, inf );
            sscanf( entry, "%s", item0 );
            tMutation.seq = [ NSString stringWithUTF8String:item0 ];
            [ tMutDomain.mutationMutAr addObject:tMutation ];
            break;
            
		case '/':
            if( strcmp( entry, "//EntryEnds" ) == 0 ) {
                endFlag = 1;
            }
            break;
            
		default:
            break;
        }
        if( endFlag == 1 ) {
            break;
        }
    }
    NSInteger i, j, numberOfRegExpMatches;
    NSError *error = NULL;
    NSRegularExpression *regex = [ NSRegularExpression regularExpressionWithPattern:@"[^AGCT]" options:NSRegularExpressionCaseInsensitive error:&error ];
    for( i = 0; i < [ tMutDomainMutAr count ]; ++i ) {
        tMutDomain = tMutDomainMutAr[ i ];
        for( j = 0; j < [ tMutDomain.mutationMutAr count ]; ++j ) {
            tMutation = tMutDomain.mutationMutAr[ j ];
            const char *before10bpC = [ tMutation.before10bp UTF8String ];
            const char *after10bpC = [ tMutation.after10bp UTF8String ];
            tMutation.seqC = ( char * )[ tMutation.seq UTF8String ];
            numberOfRegExpMatches = [ regex numberOfMatchesInString:tMutation.seq options:0 range:NSMakeRange( 0, [ tMutation.seq length ] ) ];
            if( numberOfRegExpMatches == 0 ) {
                tMutation.seqContainsWildCard = NO;
            } else {
                tMutation.seqContainsWildCard = YES;
            }
            const char *seqC = [ tMutation.seq UTF8String ];
            char *startPt = nucStrstrWildCard( seqC, before10bpC );
            char *endPt = nucStrstrWildCard( seqC, after10bpC );
            *( endPt + strlen( after10bpC ) ) = 0;
            tMutation.coreSeq = [ NSString stringWithUTF8String:startPt ];
            numberOfRegExpMatches = [ regex numberOfMatchesInString:tMutation.coreSeq options:0 range:NSMakeRange( 0, [ tMutation.coreSeq length ] ) ];
            tMutation.coreSeqC = ( char * )[ tMutation.coreSeq UTF8String ];
            if( numberOfRegExpMatches == 0 ) {
                tMutation.coreSeqContainsWildCard = NO;
            } else {
                tMutation.coreSeqContainsWildCard = YES;
            }
        }
    }
    return tMutDomainMutAr;
}

@end
