//
//  Mutation.m
//  MainRoutine
//
//  Created by 萩原 弘一 on 2013/09/02.
//  Copyright (c) 2013年 萩原 弘一. All rights reserved.
//
/* Copyright 2016 Koichi Hagiwara
 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import "MINtS analyzer.h"

@implementation Mutation
@synthesize name;
@synthesize referenceFrequencyOfTheMutantReads;
@synthesize allowableFalsePositiveRate;
@synthesize allowableFalseNegativeRate;
@synthesize minimumRequiredNumberOfInformativeReads;
@synthesize seq;
@synthesize seqC;
@synthesize seqContainsWildCard;
@synthesize coreSeq;
@synthesize coreSeqC;
@synthesize coreSeqContainsWildCard;
@synthesize before10bp;
@synthesize after10bp;
@synthesize strandRequiredToRead;
@synthesize match;
@synthesize posNegDetectedOrAlleleType;
@synthesize mutantCellRatio;
@synthesize comment;

-( id )init {
    self = [ super init ];
    if( self ) {
        match = 0;
        posNegDetectedOrAlleleType = @"";
        mutantCellRatio = @"";
        comment = @"";
    }
    return self;
}

-( id )copyWithZone:( NSZone * )zone {
    Mutation *tempCopy = [ [ [ self class ] allocWithZone:zone ] init ];
    if( tempCopy ) {
        tempCopy.name = [ name copyWithZone:zone ];
        tempCopy.referenceFrequencyOfTheMutantReads = minimumRequiredNumberOfInformativeReads;
        tempCopy.allowableFalsePositiveRate = allowableFalsePositiveRate;
        tempCopy.allowableFalseNegativeRate = allowableFalseNegativeRate;
        tempCopy.minimumRequiredNumberOfInformativeReads = minimumRequiredNumberOfInformativeReads;
        tempCopy.seq = [ seq copyWithZone:zone ];
        tempCopy.seqC = ( char * )[ tempCopy.seq UTF8String ];
        tempCopy.seqContainsWildCard = seqContainsWildCard;
        tempCopy.coreSeq = [ coreSeq copyWithZone:zone ];
        tempCopy.coreSeqC = ( char * )[ tempCopy.coreSeq UTF8String ];
        tempCopy.coreSeqContainsWildCard = coreSeqContainsWildCard;
        tempCopy.before10bp = [ before10bp copyWithZone:zone ];
        tempCopy.after10bp = [ after10bp copyWithZone:zone ];
        tempCopy.strandRequiredToRead = [ strandRequiredToRead copyWithZone:zone ];
        tempCopy.match = match;
        tempCopy.posNegDetectedOrAlleleType = [ posNegDetectedOrAlleleType copyWithZone:zone ];
        tempCopy.mutantCellRatio = [ mutantCellRatio copyWithZone:zone ];
        tempCopy.comment = [ comment copyWithZone:zone ];
    }
    return tempCopy;
}

-( void )encodeWithCoder:( NSCoder * )aCoder {
    [ aCoder encodeObject:name forKey:@"name" ];
    [ aCoder encodeDouble:referenceFrequencyOfTheMutantReads forKey:@"referenceFrequencyOfTheMutantReads" ];
    [ aCoder encodeDouble:allowableFalsePositiveRate forKey:@"allowableFalsePositiveRate" ];
    [ aCoder encodeDouble:allowableFalseNegativeRate forKey:@"allowableFalseNegativeRate" ];
    [ aCoder encodeInteger:minimumRequiredNumberOfInformativeReads forKey:@"minimumRequiredNumberOfInformativeReads" ];
    [ aCoder encodeObject:seq forKey:@"seq" ]; // seqC is derived from seq
    [ aCoder encodeBool:seqContainsWildCard forKey:@"seqContainsWildCard" ];
    [ aCoder encodeObject:coreSeq forKey:@"coreSeq" ]; // coreSeqC is derived from coreSeq
    [ aCoder encodeBool:coreSeqContainsWildCard forKey:@"coreSeqContainsWildCard" ];
    [ aCoder encodeObject:before10bp forKey:@"before10bp" ];
    [ aCoder encodeObject:after10bp forKey:@"after10bp" ];
    [ aCoder encodeObject:strandRequiredToRead forKey:@"strandRequiredToRead" ];
    [ aCoder encodeInt64:match forKey:@"match" ];
    [ aCoder encodeObject:posNegDetectedOrAlleleType forKey:@"posNegDetectedOrAlleleType" ];
    [ aCoder encodeObject:mutantCellRatio forKey:@"mutantCellRatio" ];
    [ aCoder encodeObject:comment forKey:@"comment" ];
}

-( id )initWithCoder:( NSCoder * )aDecoder {
    self = [ super init ];
    name = [ aDecoder decodeObjectForKey:@"name" ];
    referenceFrequencyOfTheMutantReads = [ aDecoder decodeDoubleForKey:@"referenceFrequencyOfTheMutantReads" ];
    allowableFalsePositiveRate = [ aDecoder decodeDoubleForKey:@"allowableFalsePositiveRate" ];
    allowableFalseNegativeRate = [ aDecoder decodeDoubleForKey:@"allowableFalseNegativeRate" ];
    minimumRequiredNumberOfInformativeReads = [ aDecoder decodeIntegerForKey:@"minimumRequiredNumberOfInformativeReads" ];
    seq = [ aDecoder decodeObjectForKey:@"seq" ];
    seqC = ( char * )[ seq UTF8String ];
    seqContainsWildCard = [ aDecoder decodeBoolForKey:@"seqContainsWildCard" ];
    coreSeq = [ aDecoder decodeObjectForKey:@"coreSeq" ];
    coreSeqC = ( char * )[ coreSeq UTF8String ];
    coreSeqContainsWildCard = [ aDecoder decodeBoolForKey:@"coreSeqContainsWildCard" ];
    before10bp = [ aDecoder decodeObjectForKey:@"before10bp" ];
    after10bp = [ aDecoder decodeObjectForKey:@"after10bp" ];
    strandRequiredToRead = [ aDecoder decodeObjectForKey:@"strandRequiredToRead" ];
    match = [ aDecoder decodeInt64ForKey:@"match" ];
    posNegDetectedOrAlleleType = [ aDecoder decodeObjectForKey:@"posNegDetectedOrAlleleType" ];
    mutantCellRatio = [ aDecoder decodeObjectForKey:@"mutantCellRatio" ];
    comment = [ aDecoder decodeObjectForKey:@"comment" ];
    return self;
}

@end
