//
//  ResultCalc.m
//  MINtS analyzer
//
//  Created by 萩原 弘一 on 2013/09/19.
//  Copyright (c) 2013年 萩原 弘一. All rights reserved.
//
/* Copyright 2016 Koichi Hagiwara
 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import "MINtS analyzer.h"
#define PI  3.14159265358979323846264
@implementation ResultCalc
@synthesize mainRoutine;

-( double )p_nor:( double )z{
    int i;
    double z2, prev, p, t;
    
    z2 = z * z;
    t = p = z * exp( -0.5 * z2 ) / sqrt( 2 * PI );
    for( i = 3; i < 200; i += 2 ) {
        prev = p;  t *= z2 / i;  p += t;
        if ( p == prev ) return 0.5 + p;
    }
    return ( z > 0 );
}

-( double )q_nor:( double )z{
    return 1 - [ self p_nor:z ];
}

-( double )noMutantReadsAtFalseNegativeThresholdForTotalNoReads:( NSInteger )numberOfRead frequencyOfTruelyMutantReads:( double )frequencyOfTruelyMutantReads falseNegativeRate:( double )falseNegativeRate {
    double mean = numberOfRead * frequencyOfTruelyMutantReads;
    double variance = numberOfRead * frequencyOfTruelyMutantReads * ( 1 - frequencyOfTruelyMutantReads );
    double standardDeviation = sqrt( variance );
    if( mean > 5 && variance > 5 ){
        long i = 0;
        double d = [ self p_nor:( ( i - mean ) / standardDeviation ) ];
//        fprintf( stderr, "%ld %f\n", i, [ self p_nor:( ( i - mean ) / standardDeviation )  ]);
        while( d < falseNegativeRate ){
            i += 1;
            d = [ self p_nor:( ( i - mean ) / standardDeviation ) ];
//            fprintf( stderr, "%ld %f\n", i, [ self p_nor:( ( i - mean ) / standardDeviation )  ]);
        }
        return i - 1;
    } else {
        long i = 0;
        double d = [ self cumBinomialToNumberOfReads:i totalNoReads:numberOfRead frequencyOfMutantReads:frequencyOfTruelyMutantReads ];
        while( d < falseNegativeRate ){
            i += 1;
            d = [ self cumBinomialToNumberOfReads:i totalNoReads:numberOfRead frequencyOfMutantReads:frequencyOfTruelyMutantReads ];
        }
        return i - 1;
    }
}

-( double )noMutantReadsAtFalsePositiveThresholdForTotalNoReads:( NSInteger )numberOfRead frequencyOfFalselyMutantReads:( double )frequencyOfFalselyMutantReads falsePositiveRate:( double )falsePositiveRate {
    double mean = numberOfRead * frequencyOfFalselyMutantReads;
    double variance = numberOfRead * frequencyOfFalselyMutantReads * ( 1 - frequencyOfFalselyMutantReads );
    double standardDeviation = sqrt( variance );

    if( mean > 5 && variance > 5 ){
        long i = 0;
        double d = [ self p_nor:( ( i - mean ) / standardDeviation ) ];
//        fprintf( stderr, "%ld %f\n", i, [ self p_nor:( ( i - mean ) / standardDeviation )  ]);
        while( d < 1 - falsePositiveRate ){
            i += 1;
            d = [ self p_nor:( ( i - mean ) / standardDeviation ) ];
//            fprintf( stderr, "%ld %f\n", i, [ self p_nor:( ( i - mean ) / standardDeviation )  ]);
        }
        return i - 1;
    } else {
        long i = 0;
        double d = [ self cumBinomialToNumberOfReads:i totalNoReads:numberOfRead frequencyOfMutantReads:frequencyOfFalselyMutantReads ];
        while( d < 1 - falsePositiveRate ){
            i += 1;
            d = [ self cumBinomialToNumberOfReads:i totalNoReads:numberOfRead frequencyOfMutantReads:frequencyOfFalselyMutantReads ];
        }
        return i;
    }
}

-( double )cumBinomialToNumberOfReads:( NSInteger )k totalNoReads:( NSInteger )n frequencyOfMutantReads:( double )p {
    long i;
    double d = 0.0, dif = 0.0;
    for( i = 0; i <= k; ++i ){
        dif = [ self binomialValueAt:i totalNoReads:n frequencyOfMutantReads:p ];
        if( ( n + 1 ) * p < i && dif < 0.0000000001 ) break;
        d = d + dif;
    }
    return d;
}

-( double )binomialValueAt:( NSInteger )k totalNoReads:( NSInteger )n frequencyOfMutantReads:( double )p {
    if( k == 0 ) {
        return pow( ( 1 - p ), ( double )n );
    }
    else {
        return [ self binomialValueAt:( k - 1 ) totalNoReads:n frequencyOfMutantReads:p ] * ( ( n - k + 1 ) / ( double )k ) * ( p / ( 1 - p ) );
    }
}

//Summarize reads info stored into mutdomain and define wt, mut, positive or negative
-( void )calculateResult:( MutDomain * )mutdomain {
    NSInteger nD1Threshold, nD2Threshold;
    bool calculateFalsePositiveRate;
    NSUserDefaults *defaults = [ NSUserDefaults standardUserDefaults ];
    nD1Threshold = [ [ defaults valueForKey:@"nD1Threshold" ] integerValue ];
    nD2Threshold = [ [ defaults valueForKey:@"nD2Threshold" ] integerValue ];
    calculateFalsePositiveRate = [ [ defaults valueForKey:@"calculateFalsePositiveRate" ] boolValue ];
    if( nD1Threshold <= 0 ) {
        nD1Threshold = MIN_NUMBER_OF_READS_REQUIRED_FOR_SOMATIC_DNA;
        [ defaults setValue:[ NSNumber numberWithInteger:nD1Threshold ] forKey:@"nD1Threshold" ];
    }
    if( nD2Threshold <= 0 ) {
        nD2Threshold = MIN_NUMBER_OF_READS_REQUIRED_FOR_FUSION_MRNA;
        [ defaults setValue:[ NSNumber numberWithInteger:nD2Threshold ] forKey:@"nD2Threshold" ];
    }
    NSInteger i;
    if( [ mutdomain.type isEqualToString:@"Germline" ] ) {
        if( [ mutdomain.name isEqualToString:@"BIM" ] ) {
            NSInteger wildType, mutant;
            Mutation *tMutation = mutdomain.mutationMutAr[ 0 ];
            wildType = tMutation.match;
            tMutation = mutdomain.mutationMutAr[ 1 ];
            mutant = tMutation.match;
            tMutation = mutdomain.mutationMutAr[ 0 ];
            if( wildType < 100 && mutant < 100 ) {
                tMutation = mutdomain.mutationMutAr[ 0 ];
                tMutation.posNegDetectedOrAlleleType = @"Not Determined";
            } else if( mutant * 100 < wildType ) {
                tMutation.posNegDetectedOrAlleleType = @"wt/wt";
            } else if( mutant > wildType * 10 ) {
                tMutation.posNegDetectedOrAlleleType = @"mut/mut";
            } else {
                tMutation.posNegDetectedOrAlleleType = @"wt/mut";
            }
            tMutation.mutantCellRatio = @"";
            tMutation.comment = @"";
        }
    }
    if( [ mutdomain.type isEqualToString:@"SomaticDNA" ] ) {
        for( i = 1; i < [ mutdomain.mutationMutAr count ]; ++i ) {
            Mutation *tMutation = mutdomain.mutationMutAr[ i ];
            NSInteger totalNumberOfReads = mutdomain.totalNumberOfNormalReads + tMutation.match;
            if( totalNumberOfReads < nD1Threshold ) {
                tMutation.posNegDetectedOrAlleleType = @"ND *1";
                tMutation.mutantCellRatio = @"";
                tMutation.comment = @"";
                continue;
            }
            NSInteger mutantReads = tMutation.match;
            NSInteger noMutantReadsAtFalseNegativeThreshold = [ self noMutantReadsAtFalseNegativeThresholdForTotalNoReads:totalNumberOfReads frequencyOfTruelyMutantReads:0.005 falseNegativeRate:tMutation.allowableFalseNegativeRate ];
            NSInteger noMutantReadsAtFalsePositiveThreshold = [ self noMutantReadsAtFalsePositiveThresholdForTotalNoReads:totalNumberOfReads frequencyOfFalselyMutantReads:tMutation.referenceFrequencyOfTheMutantReads falsePositiveRate:tMutation.allowableFalsePositiveRate ];
            // If total reads are too few, do not evaluate.
            if( mutantReads <= noMutantReadsAtFalseNegativeThreshold ) {
                tMutation.posNegDetectedOrAlleleType = @"Negative";
                tMutation.mutantCellRatio = @"";
                tMutation.comment = @"";
                continue;
            }
            if( mutantReads >= noMutantReadsAtFalsePositiveThreshold ) {
                tMutation.posNegDetectedOrAlleleType = @"Positive";
                double tMutantCellRatio = ( double )mutantReads / totalNumberOfReads * 100.0 * 2.0;
                if( tMutantCellRatio >= 100.0 ) {
                    tMutantCellRatio = 100.0;
                }
                tMutation.mutantCellRatio = [ NSString stringWithFormat:@"%0.1f%%", tMutantCellRatio ];
                if( calculateFalsePositiveRate == 1 ) {
                    double tFalsePositiveRate = 1.0 - [ self cumBinomialToNumberOfReads:mutantReads totalNoReads:totalNumberOfReads frequencyOfMutantReads:tMutation.referenceFrequencyOfTheMutantReads ];
                    if( tFalsePositiveRate < 0 ) { tFalsePositiveRate = -tFalsePositiveRate; }
                    tMutation.comment = [ NSString stringWithFormat:@"%0.10f", tFalsePositiveRate ];
                } else {
                    tMutation.comment = @"";
                }
                continue;
            }
            if( totalNumberOfReads < tMutation.minimumRequiredNumberOfInformativeReads ) {
                tMutation.posNegDetectedOrAlleleType = @"Not Determined";
                tMutation.mutantCellRatio = @"";
                tMutation.comment = @"";
                continue;
            }
        }
    }
    if( [ mutdomain.type isEqualToString:@"germline_mRNA" ] ) {
        Mutation *tMutation = mutdomain.mutationMutAr[ 0 ];
        if( tMutation.match == 0 ) {
            tMutation.posNegDetectedOrAlleleType = @"NotDetected";
        } else {
            tMutation.posNegDetectedOrAlleleType = @"Detected";
        }
        tMutation.mutantCellRatio = @"";
        tMutation.comment = @"";
    }
    if( [ mutdomain.type isEqualToString:@"fusion_RNA" ] ) {
        Mutation *tMutation = mutdomain.mutationMutAr[ 0 ];
        if( [ tMutation.name rangeOfString:@"control" ].location != NSNotFound ) { // Fusion having control
            NSInteger controlExistFlag = 0, mutantExistFlag = 0;
            if( tMutation.match != 0 ) {
                tMutation.posNegDetectedOrAlleleType = @"Detected";
                controlExistFlag = 1;
            } else {
                tMutation.posNegDetectedOrAlleleType = @"NotDetected";
            }
            for( i = 1; i < [ mutdomain.mutationMutAr count ]; ++i ) {
                tMutation = mutdomain.mutationMutAr[ i ];
                if( tMutation.match == 0 ) {
                    tMutation.posNegDetectedOrAlleleType = @"Negative";
                } else {
                    tMutation.posNegDetectedOrAlleleType = @"Positive";
                    mutantExistFlag = 1;
                }
            }
            if( mutantExistFlag == 1 || controlExistFlag == 0 ) {
                tMutation.comment = @"";
            } else {
                tMutation.comment = @"copies"; // under construction
            }
        } else {    // Fusion not having control
            for( i = 0; i < [ mutdomain.mutationMutAr count ]; ++i ) {
                tMutation = mutdomain.mutationMutAr[ i ];
                if( tMutation.match == 0 ) {
                    tMutation.posNegDetectedOrAlleleType = @"Negative";
                } else {
                    tMutation.posNegDetectedOrAlleleType = @"Positive";
                }
            }
        }
    }
}

//Information on reads is summarized in mutDomains.
//Summarize result into gemlineDNAResultMutAr, somaticDNAResultMutAr, germlineRNAResultMutAr and somaticRNAResultMutAr and attach to summaryForEachPatient.
-( void )summarizeResultForPatient:( NSString * )tPtNo mutDomains:( NSMutableArray * )tMutDomains summaryForEachPatient:( SummaryForEachPatient * )summaryForEachPatient {
    NSInteger i, j;
    bool firstDNAEntry = YES;
    bool firstFusionMRNAEntry = YES;
    for( i = 0; i < [ tMutDomains count ]; ++i ) {
        MutDomain *tMutdomain = tMutDomains[ i ];
        Mutation *tMutation;
        if( [ tMutdomain.type isEqualToString:@"Germline" ] ) {
            if( [ tMutdomain.name isEqualToString:@"BIM" ] ) {
                Mutation *tMutation = tMutdomain.mutationMutAr[ 0 ];
                ReportGermlineDNA *tReportGermlineDNA = [ [ ReportGermlineDNA alloc ] init ];
                tReportGermlineDNA.mutDomain = tMutdomain;
                if( firstDNAEntry == YES ) {
                    tReportGermlineDNA.patientNo = tPtNo;
                    firstDNAEntry = NO;
                } else {
                    tReportGermlineDNA.patientNo = @"";
                }
                tReportGermlineDNA.areaSearched = tMutdomain.name;
                tReportGermlineDNA.allelicType = tMutation.posNegDetectedOrAlleleType;
                summaryForEachPatient.germlineDNAResultMutAr = [ NSMutableArray arrayWithCapacity:1 ];
                [ summaryForEachPatient.germlineDNAResultMutAr addObject:tReportGermlineDNA ];
            }
        }
        if( [ tMutdomain.type isEqualToString:@"SomaticDNA" ] ) {
            bool mutationPositive = NO;
            for( j = 1; j < [ tMutdomain.mutationMutAr count ]; ++j ) {
                Mutation *tMutation = tMutdomain.mutationMutAr[ j ];
                if( [ tMutation.posNegDetectedOrAlleleType isEqualToString:@"Positive" ] ) {
                    mutationPositive = YES;
                    ReportSomaticDNA *tReportSomaticDNA = [ [ ReportSomaticDNA alloc ] init ];
                    tReportSomaticDNA.mutDomain = tMutdomain;
                    if( firstDNAEntry == YES ) {
                        tReportSomaticDNA.patientNo = tPtNo;
                        firstDNAEntry = NO;
                    } else {
                        tReportSomaticDNA.patientNo = @"";
                    }
                    tReportSomaticDNA.areaSearched = tMutation.name;
                    tReportSomaticDNA.diagnosis = @"Positive";
                    tReportSomaticDNA.mutantCellRatio = tMutation.mutantCellRatio;
                    tReportSomaticDNA.comment = tMutation.comment;
                    [ summaryForEachPatient.somaticDNAResultMutAr addObject:tReportSomaticDNA ];
                }
            }
            if( mutationPositive == NO ) {
                ReportSomaticDNA *tReportSomaticDNA = [ [ ReportSomaticDNA alloc ] init ];
                tReportSomaticDNA.mutDomain = tMutdomain;
                if( firstDNAEntry == YES ) {
                    tReportSomaticDNA.patientNo = tPtNo;
                    firstDNAEntry = NO;
                } else {
                    tReportSomaticDNA.patientNo = @"";
                }
                tReportSomaticDNA.areaSearched = tMutdomain.name;
                tReportSomaticDNA.diagnosis = @"Negative";
                tReportSomaticDNA.mutantCellRatio = @"";
                
                double tDetectionLimit = 0.0;
                NSString *tDetectionLimitSt = @"";
                for( j = 1; j < [ tMutdomain.mutationMutAr count ]; ++j ) {
                    Mutation *tMutation = tMutdomain.mutationMutAr[ j ];
                    if( [ tMutation.posNegDetectedOrAlleleType isEqualToString:@"NDNegative" ] ) { // Reads too few
                        tReportSomaticDNA.diagnosis = @"NDNegative";
                        break;
                    } else if( [ tMutation.posNegDetectedOrAlleleType isEqualToString:@"ND *0" ] ) { // Reads too few
                        tReportSomaticDNA.diagnosis = @"Too few reads";
                         break;
                    } else if( [ tMutation.posNegDetectedOrAlleleType isEqualToString:@"ND *1" ] ) { // Total reads < ND1 threshold
                        tReportSomaticDNA.diagnosis = @"ND *1";
                        break;
                    } else if( [ tMutation.posNegDetectedOrAlleleType isEqualToString:@"ND *2" ] ) { // Even positive, # of positive reads < ND2 threshold
                        tReportSomaticDNA.diagnosis = @"ND *2";
                        break;
                    } else {
                        if( [ tMutation.comment doubleValue ] > tDetectionLimit ) {
                            tDetectionLimit = [ tMutation.comment doubleValue ];
                            tDetectionLimitSt = tMutation.comment; // Record poorest detection limit
                        }
                    }
                }
                if( j == [ tMutdomain.mutationMutAr count ] ) {
                    tReportSomaticDNA.comment = tDetectionLimitSt;
                } else {
                    tReportSomaticDNA.comment = @"";
                }
                [ summaryForEachPatient.somaticDNAResultMutAr addObject:tReportSomaticDNA ];
            }
        }
        if( [ tMutdomain.type isEqualToString:@"germline_mRNA" ] ) {
            ReportGermlineRNA *tReportGermlineRNA = [ [ ReportGermlineRNA alloc ] init ];
            tReportGermlineRNA.mutDomain = tMutdomain;
            tReportGermlineRNA.patientNo = @"";
            tReportGermlineRNA.gene = tMutdomain.name;
            tMutation = tMutdomain.mutationMutAr[ 0 ];   // First entry of Mutations
            if( [ tMutation.name isEqualToString:@"OAZ1" ] ) {
                if( tMutation.match == 0 ) {
                    tReportGermlineRNA.diagnosis = @"RNA degraded";
                    tReportGermlineRNA.numberOfReads = @"0";
                    tReportGermlineRNA.comment = @"";
                } else {
                    tReportGermlineRNA.diagnosis = @"RNA not degraded";
                    tReportGermlineRNA.numberOfReads = [ NSString stringWithFormat:@"%ld",tMutation.match ];
                    tReportGermlineRNA.comment = @"";
                }
                summaryForEachPatient.germlineRNAResultMutAr = [ NSMutableArray arrayWithCapacity:1 ];
                [ summaryForEachPatient.germlineRNAResultMutAr addObject:tReportGermlineRNA ];
            }
        }
        if( [ tMutdomain.type isEqualToString:@"fusion_RNA" ] ) {
            bool mutationPositive = NO;
            for( j = 0; j < [ tMutdomain.mutationMutAr count ]; ++j ) {
                tMutation = tMutdomain.mutationMutAr[ j ];
                if( [ tMutation.posNegDetectedOrAlleleType isEqualToString:@"Positive" ] ) {
                    mutationPositive = YES;
                    ReportSomaticRNA *tReportSomaticRNA = [ [ ReportSomaticRNA alloc ] init ];
                    tReportSomaticRNA.mutDomain = tMutdomain;
                    if( firstFusionMRNAEntry == YES ) {
                        tReportSomaticRNA.patientNo = tPtNo;
                        firstFusionMRNAEntry = NO;
                    } else {
                        tReportSomaticRNA.patientNo = @"";
                    }
                    tReportSomaticRNA.gene = tMutation.name;
                    tReportSomaticRNA.diagnosis = @"Positive";
                    tReportSomaticRNA.numberOfReads = [ NSString stringWithFormat:@"%ld",tMutation.match ];
                    tReportSomaticRNA.comment = @"";
                    [ summaryForEachPatient.somaticRNAResultMutAr addObject:tReportSomaticRNA ];
                }
            }
            if( mutationPositive == NO ) {
                ReportSomaticRNA *tReportSomaticRNA = [ [ ReportSomaticRNA alloc ] init ];
                tReportSomaticRNA.mutDomain = tMutdomain;
                if( firstFusionMRNAEntry == YES ) {
                    tReportSomaticRNA.patientNo = tPtNo;
                    firstFusionMRNAEntry = NO;
                } else {
                    tReportSomaticRNA.patientNo = @"";
                }
                tReportSomaticRNA.gene = tMutdomain.name;
                tMutation = tMutdomain.mutationMutAr[ 0 ];   // First entry of Mutations
                if( [ tMutation.name isEqualToString:@"OAZ1" ] ) {
                    if( tMutation.match == 0 ) {
                        tReportSomaticRNA.diagnosis = @"RNA degraded";
                        tReportSomaticRNA.numberOfReads = @"0";
                        tReportSomaticRNA.comment = @"";
                    } else {
                        tReportSomaticRNA.diagnosis = @"RNA not degraded";
                        tReportSomaticRNA.numberOfReads = [ NSString stringWithFormat:@"%ld",tMutation.match ];
                        tReportSomaticRNA.comment = @"";
                    }
                } else if( [ tMutation.name rangeOfString:@"control" ].location == NSNotFound ) {
                    tReportSomaticRNA.diagnosis = @"Not Detected";
                    tReportSomaticRNA.numberOfReads = @"0";
                    tReportSomaticRNA.comment = @"";
                } else {
                    if( tMutation.match == 0 ) {
                        tReportSomaticRNA.diagnosis = @"Control Not Working";
                        tReportSomaticRNA.numberOfReads = @"";
                        tReportSomaticRNA.comment = @"";
                    } else {
                        tReportSomaticRNA.diagnosis = @"Negative";
                        tReportSomaticRNA.numberOfReads = [ NSString stringWithFormat:@"%ld",tMutation.match ];
                        double probabilityOfControl = exp( ( log( 0.01 ) / tMutation.match ) );
                        NSInteger inputControlCopyNumber = [ [ [ NSUserDefaults standardUserDefaults ] valueForKey:@"copyNumberOfInternalControl" ] integerValue ];
                        NSInteger mutantCopyNumberLimit = inputControlCopyNumber * ( 1.0 - probabilityOfControl );
                        tReportSomaticRNA.comment = [ NSString stringWithFormat:@"<%ld copies", mutantCopyNumberLimit ];
                    }
                }
                [ summaryForEachPatient.somaticRNAResultMutAr addObject:tReportSomaticRNA ];
            }
        }
    }
}

-( id )init {
    self = [ super init ];
    if( self ) {
    }
    return self;
}

@end
