//
//  WindowController.m
//  MINtS analyzer
//
//  Created by 萩原 弘一 on 2013/09/05.
//  Copyright (c) 2013年 萩原 弘一. All rights reserved.
//
/* Copyright 2016 Koichi Hagiwara
 This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import "MINtS analyzer.h"

@interface WindowController ()

@end

@implementation WindowController
@synthesize mintsDocument;
@synthesize mainRoutine;
@synthesize mainWindow;
@synthesize file1TextField;
@synthesize file2TextField;
@synthesize totalNumberOfReadsTextField;
@synthesize totalNumberOfIntormativeReadsTextField;
@synthesize readsMappedTextField;
@synthesize readsFailedToBeMappedTextField;
@synthesize germlineDNAResultTableView;
@synthesize somaticDNAResultTableView;
@synthesize germlineRNAResultTableView;
@synthesize somaticRNAResultTableView;
@synthesize germlineDNAResultTextField;
@synthesize somaticDNAResultTextField;
@synthesize germlineRNAResultTextField;
@synthesize somaticRNAResultTextField;
@synthesize analyzeButton;
@synthesize analyzeAllButton;
@synthesize indicator;
@synthesize totalNumberOfReads;
@synthesize totalNumberOfOverlappedReads;
@synthesize readsMapped;
@synthesize readsFailedToBeMapped;
@synthesize errorLogTextView;
@synthesize errorLogWindow;
@synthesize patientsNoTableView;
@synthesize popoverViewControllerHUD;
@synthesize popover;
@synthesize summaryForEachPatientUsedForPopover;
@synthesize mutDomainUsedForPopover;
@synthesize popoverTableView;
@synthesize popoverCommentTextField;


-( BOOL )validateMenuItem:( NSMenuItem * )anItem {
    if( [ [ anItem title ] rangeOfString:@"Show Error Log" ].location != NSNotFound ) {
        return YES;
    } else if( [ [ anItem title ] rangeOfString:@"Show ND1 and ND2 Threshold" ].location != NSNotFound ) {
        return YES;
    } else if( [ [ anItem title ] rangeOfString:@"Copy" ].location != NSNotFound ) {
        if( [ [ patientsNoTableView selectedRowIndexes ] count ] != 0 ) {
            return YES;
        } else {
            return NO;
        }
    }
    return [ super validateMenuItem:anItem ];
}

-( void )showErrorLogWindow:( id )sender {
    [ errorLogWindow makeKeyAndOrderFront:errorLogWindow ];
}

-( void )tableViewSelectionDidChange:( NSNotification * ) notification {
    NSTableView *tableView = [ notification object ];	// notification is a NSTableView
    
    if ( tableView == patientsNoTableView ){
        if( [ [ tableView selectedRowIndexes ] count ] <= 0 ) { return; }
        mintsDocument.mainRoutine.selectedPatientRowNo = [ tableView selectedRow ];
        SummaryForEachPatient *tSummaryForEachPatient = mintsDocument.mainRoutine.summaryForEachPatientMutAr[ mintsDocument.mainRoutine.selectedPatientRowNo ];
        [ file1TextField setStringValue:tSummaryForEachPatient.file1 ];
        [ file2TextField setStringValue:tSummaryForEachPatient.file2 ];
        [ totalNumberOfReadsTextField setStringValue:[ NSString stringWithFormat:@"%ld", tSummaryForEachPatient.totalNumberOfReads ] ];
        [ readsMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", tSummaryForEachPatient.readsMapped ] ];
        [ readsFailedToBeMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", tSummaryForEachPatient.readsFailedToBeMapped ] ];
        NSInteger setAlert = 0;
        for( int j = 0; j < [ tSummaryForEachPatient.germlineDNAResultMutAr count ]; ++j ) {
            ReportGermlineDNA *tReportGermlineDNA = tSummaryForEachPatient.germlineDNAResultMutAr[ j ];
            NSString *tAllelicType = tReportGermlineDNA.allelicType;
            if( [ tAllelicType isEqualToString:@"mut/mut" ] || [ tAllelicType isEqualToString:@"wt/mut" ] || [ tAllelicType isEqualToString:@"Not Determined" ] ) {
                setAlert = 1;
            }
        }
        if( setAlert == 1 ) {
            [ germlineDNAResultTextField setHidden:NO ];
        } else {
            [ germlineDNAResultTextField setHidden:YES ];
        }
        setAlert = 0;
        for( int j = 0; j < [ tSummaryForEachPatient.somaticDNAResultMutAr count ]; ++j ) {
            ReportSomaticDNA *tReportSomaticDNA = tSummaryForEachPatient.somaticDNAResultMutAr[ j ];
            NSString *tDiagnosis = tReportSomaticDNA.diagnosis;
            if( [ tDiagnosis isEqualToString:@"Positive" ] ) {
                setAlert = 1;
            }
        }
        if( setAlert == 1 ) {
            [ somaticDNAResultTextField setHidden:NO ];
        } else {
            [ somaticDNAResultTextField setHidden:YES ];
        }
        setAlert = 0;
        for( int j = 0; j < [ tSummaryForEachPatient.germlineRNAResultMutAr count ]; ++j ) {
            ReportGermlineRNA *tReportGermlineRNA = tSummaryForEachPatient.germlineRNAResultMutAr[ j ];
            NSString *tDiagnosis = tReportGermlineRNA.diagnosis;
            if( [ tDiagnosis isEqualToString:@"RNA degraded" ] ) {
                setAlert = 1;
            }
        }
        if( setAlert == 1 ) {
            [ germlineRNAResultTextField setHidden:NO ];
        } else {
            [ germlineRNAResultTextField setHidden:YES ];
        }
        [ germlineDNAResultTableView reloadData ];
        [ somaticDNAResultTableView reloadData ];
        [ germlineRNAResultTableView reloadData ];
        [ somaticRNAResultTableView reloadData ];
        setAlert = 0;
        for( int j = 0; j < [ tSummaryForEachPatient.somaticRNAResultMutAr count ]; ++j ) {
            ReportSomaticRNA *tReportSomaticRNA = tSummaryForEachPatient.somaticRNAResultMutAr[ j ];
            NSString *tDiagnosis = tReportSomaticRNA.diagnosis;
            if( [ tDiagnosis isEqualToString:@"Positive" ] ) {
                setAlert = 1;
            }
        }
        if( setAlert == 1 ) {
            [ somaticRNAResultTextField setHidden:NO ];
        } else {
            [ somaticRNAResultTextField setHidden:YES ];
        }
        [ germlineDNAResultTableView reloadData ];
        [ somaticDNAResultTableView reloadData ];
        [ germlineRNAResultTableView reloadData ];
        [ somaticRNAResultTableView reloadData ];
    }
}

-( NSInteger )numberOfRowsInTableView:( NSTableView * )aTableView {
    if( aTableView == patientsNoTableView ) {
        return [ mintsDocument.mainRoutine.summaryForEachPatientMutAr count ];
    } else if( aTableView == popoverTableView ) {
        return( [ mutDomainUsedForPopover.mutationMutAr count ] );
    } else {
        NSMutableArray *tSummaryOfReadDataMutAr = mintsDocument.mainRoutine.summaryForEachPatientMutAr;
        if( [ tSummaryOfReadDataMutAr count ] == 0 ) return 0;
        SummaryForEachPatient *tSummaryForEachPatient = mintsDocument.mainRoutine.summaryForEachPatientMutAr[ mintsDocument.mainRoutine.selectedPatientRowNo ];
        if( aTableView == germlineDNAResultTableView ) {
            return [ tSummaryForEachPatient.germlineDNAResultMutAr count ];
        }
        if( aTableView == somaticDNAResultTableView ) {
            return [ tSummaryForEachPatient.somaticDNAResultMutAr count ];
        }
        if( aTableView == germlineRNAResultTableView ) {
            return [ tSummaryForEachPatient.germlineRNAResultMutAr count ];
        }
        if( aTableView == somaticRNAResultTableView ) {
            return [ tSummaryForEachPatient.somaticRNAResultMutAr count ];
        }
    }
    return 0; // Not reach here.
}

-( id )tableView:( NSTableView * )aTableView objectValueForTableColumn:( NSTableColumn * )aTableColumn row:( NSInteger )rowIndex {
    if( aTableView == patientsNoTableView ) {
        SummaryForEachPatient *tSummaryForEachPatient = mintsDocument.mainRoutine.summaryForEachPatientMutAr[ rowIndex ];
        return tSummaryForEachPatient.patientNo;
    } else if( aTableView == popoverTableView ) {
        Mutation *tMutation = mutDomainUsedForPopover.mutationMutAr[ rowIndex ];
        if( [ [ aTableColumn identifier ] isEqualToString:@"0" ] ) {
            return tMutation.name;
        } else if( [ [ aTableColumn identifier ] isEqualToString:@"1" ] ) {
            return [ NSString stringWithFormat:@"%ld", tMutation.match ];
        }
    } else {
        SummaryForEachPatient *tSummaryForEachPatient = mintsDocument.mainRoutine.summaryForEachPatientMutAr[ mintsDocument.mainRoutine.selectedPatientRowNo ];
        if( aTableView == germlineDNAResultTableView ) {
            ReportGermlineDNA *tReportGermlineDNA = tSummaryForEachPatient.germlineDNAResultMutAr[ rowIndex ];
            if( [ [ aTableColumn identifier ] isEqualToString:@"0" ] ) {
                return tReportGermlineDNA.areaSearched;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"1" ] ) {
                return tReportGermlineDNA.allelicType;
            }
        } else if( aTableView == somaticDNAResultTableView ) {
            ReportSomaticDNA *tReportSomaticDNA = tSummaryForEachPatient.somaticDNAResultMutAr[ rowIndex ];
            if( [ [ aTableColumn identifier ] isEqualToString:@"0" ] ) {
                return tReportSomaticDNA.areaSearched;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"1" ] ) {
                return tReportSomaticDNA.diagnosis;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"2" ] ) {
                return tReportSomaticDNA.mutantCellRatio;
            }
        } else if( aTableView == germlineRNAResultTableView ) {
            ReportGermlineRNA *tReportGermlineRNA = tSummaryForEachPatient.germlineRNAResultMutAr[ rowIndex ];
            if( [ [ aTableColumn identifier ] isEqualToString:@"0" ] ) {
                return tReportGermlineRNA.gene;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"1" ] ) {
                return tReportGermlineRNA.diagnosis;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"2" ] ) {
                return tReportGermlineRNA.numberOfReads;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"3" ] ) {
                return tReportGermlineRNA.comment;
            }
        } else if( aTableView == somaticRNAResultTableView ) {
            ReportSomaticRNA *tReportSomaticRNA = tSummaryForEachPatient.somaticRNAResultMutAr[ rowIndex ];
            if( [ [ aTableColumn identifier ] isEqualToString:@"0" ] ) {
                return tReportSomaticRNA.gene;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"1" ] ) {
                return tReportSomaticRNA.diagnosis;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"2" ] ) {
                return tReportSomaticRNA.numberOfReads;
            } else if( [ [ aTableColumn identifier ] isEqualToString:@"3" ] ) {
                return tReportSomaticRNA.comment;
            }
        }
    }
    return nil; // Not reach here.
}

-( void )tableView:( NSTableView * )tableView willDisplayCell:( id )cell forTableColumn:( NSTableColumn * )tableColumn row:( int )row {
    if( tableView == somaticDNAResultTableView || tableView == somaticDNAResultTableView || tableView == somaticRNAResultTableView || tableView == germlineRNAResultTableView ) {
        if( [ [ cell stringValue ] isEqualToString:@"Negative" ] ) {
            [ cell setTextColor: [ NSColor blueColor ] ];
        } else if( [ [ cell stringValue ] isEqualToString:@"Positive" ] ) {
            [ cell setTextColor: [ NSColor redColor ] ];
        } else if( [ [ cell stringValue ] isEqualToString:@"RNA degraded" ] ) {
            [ cell setTextColor: [ NSColor redColor ] ];
        } else if( [ [ cell stringValue ] isEqualToString:@"RNA not degraded" ] ) {
            [ cell setTextColor: [ NSColor blueColor ] ];
        } else if( [ [ cell stringValue ] isEqualToString:@"Control Not Working" ] ) {
            [ cell setTextColor: [ NSColor orangeColor ] ];
        } else if( [ [ cell stringValue ] isEqualToString:@"Not Detected" ] ) {
            [ cell setTextColor: [ NSColor blueColor ] ];
        } else {
            [ cell setTextColor: [ NSColor blackColor ] ];
        }
    }
}

-( IBAction )doubleClickAction:( id )sender {
    [ mintsDocument doubleClickAction:( id )sender ];
}

-( void )analyze:( id )sender {
    [ mintsDocument analyze:( id )sender ];
}

-( void )analyzeAll:( id )sender {
    [ mintsDocument analyzeAll:( id )sender ];
}

-( void )copy:( id )sender {
    [ mintsDocument copy:( id )sender ];
}

-( void )cancelOperation:( id )sender {
    mintsDocument.cancelOperationFlag = YES;
}

-( id )initWithWindow:( NSWindow * )window {
    self = [ super initWithWindow:window ];
    if( self ) {
    }
    return self;
}

-( void )updateWindowFromSummaryForEachPatient:( SummaryForEachPatient * )tSummaryForEachPatient {
    [ file1TextField setStringValue:tSummaryForEachPatient.file1 ];
    [ file2TextField setStringValue:tSummaryForEachPatient.file2 ];
    [ totalNumberOfReadsTextField setStringValue:[ NSString stringWithFormat:@"%ld", tSummaryForEachPatient.totalNumberOfReads ] ];
    [ readsMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", tSummaryForEachPatient.readsMapped ] ];
    [ readsFailedToBeMappedTextField setStringValue:[ NSString stringWithFormat:@"%ld", tSummaryForEachPatient.readsFailedToBeMapped ] ];
    if( mainRoutine.selectedPatientRowNo >= 0 ) {
        NSIndexSet *indexSet = [ [ NSIndexSet alloc ] initWithIndex:( NSUInteger )mainRoutine.selectedPatientRowNo ];
        [ patientsNoTableView selectRowIndexes:indexSet byExtendingSelection:NO ];
    }
}


-( void )windowDidLoad {
    [ super windowDidLoad ];

    NSColor *windowBackgroundColor = self.window.backgroundColor;   // Obtain window's background
    [ patientsNoTableView setBackgroundColor:windowBackgroundColor ];
    
    indicator = NO;
    
    NSUserDefaults *defaults = [ NSUserDefaults standardUserDefaults ];
    NSString *copyNumberOfInternalControl = [ defaults valueForKey:@"copyNumberOfInternalControl" ];
    if( copyNumberOfInternalControl == nil ) {
        [ defaults setValue:@"100000" forKey:@"copyNumberOfInternalControl" ];
    }
    [ self updateWindowFromSummaryForEachPatient:mintsDocument.mainRoutine.summaryForEachPatient ];
    if( [ mintsDocument.fileType isEqualToString:@"MINtSFile" ] ) {
        [ analyzeAllButton setEnabled:NO ];
        [ analyzeButton setEnabled:NO ];
    }
    popover = [ [ NSPopover alloc ] init ];
    popover.contentViewController = popoverViewControllerHUD;
    popover.appearance = 0;
    popover.animates = YES;
    popover.behavior = NSPopoverBehaviorTransient;
}

@end
